require 'graphql/client'
require 'graphql/client/http'
require 'uri'

class Specular::Integration::GitHub < Specular::Integration::Base
  handler :github
  arguments :github_token, :repo

  def initialize(github_token:, repo:)
    @https = GraphQL::Client::HTTP.new('https://api.github.com/graphql') do
      define_method(:headers) do |_context|
        {
          "User-Agent": "Specular/#{Specular.version}",
          "Authorization": "Bearer #{github_token}"
        }
      end
    end

    @repo = repo

    @schema = GraphQL::Client.load_schema(@https)
    @client = GraphQL::Client.new(schema: @schema, execute: @https)

    @client.allow_dynamic_queries = true
  end

  def repository_query
    @repository_query ||= @client.parse <<-'GRAPHQL'
      query ($name: String!, $owner: String!) {
        repository(name: $name, owner: $owner) {
          id
        }
      }
    GRAPHQL
  end

  def issue_mutation
    @issue_mutation ||= @client.parse <<-'GRAPHQL'
      mutation ($input: CreateIssueInput!) {
        createIssue(input: $input) {
          issue {
            url
          }
        }
      }
    GRAPHQL
  end

  def repository_id
    owner, name = URI.parse(@repo).path.split('/')[1,2]

    result = @client.query(
      self.repository_query,
      variables: {
        name: name,
        owner: owner
      }
    )

    result.data.repository.id
  end

  def create_issue!(title:, body:)
    result = @client.query(
      self.issue_mutation,
      variables: {
        input: {
          title: title,
          body: body,
          repositoryId: self.repository_id
        }
      }
    )

    {
      url: result.data.create_issue.issue.url
    }
  end
end
